#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @File    :   fedhf\api\opt\__init__.py
# @Time    :   2022-05-03 15:58:49
# @Author  :   Bingjie Yan
# @Email   :   bj.yan.pa@qq.com
# @License :   Apache License 2.0

__all__ = ["opts", "Config"]

from .opts import opts
from .cfg import Config
